@@ -11,8 +11,8 @@ from trexmodel.models.datastore.user_models import UserMin
 import trexmodel.conf as model_conf
 from trexlib.utils.security_util import generate_user_id, hash_password
 from trexlib.utils.string_util import random_number, is_empty, is_not_empty,\
-    split_by_length
-import logging
+    split_by_length, random_string
+import logging, hashlib
 from datetime import datetime, timedelta
 from trexlib.utils.common.date_util import parse_datetime
 from trexmodel import conf, program_conf
@@ -52,19 +52,25 @@ class MerchantAcct(MerchantMin):
     logo_storage_filename                   = ndb.StringProperty(required=False)
     dashboard_stat_figure                   = ndb.JsonProperty()
     currency_code                           = ndb.StringProperty(required=False, default='myr')
+    api_key                                 = ndb.StringProperty(required=False)
+    
     
     published_program_configuration         = ndb.JsonProperty()
     published_voucher_configuration         = ndb.JsonProperty()
     membership_configuration                = ndb.JsonProperty()
     tier_membership_configuration           = ndb.JsonProperty()
     
+    reward_naming_configuration             = ndb.JsonProperty()
+    
+    prepaid_configuration                   = ndb.JsonProperty()
+    
     stat_figure_update_interval_in_minutes  = conf.MERCHANT_STAT_FIGURE_UPDATE_INTERVAL_IN_MINUTES
     stat_figure_update_datetime_format      = '%d-%m-%Y %H:%M:%S'
     
     dict_properties = ['company_name', 'contact_name', 'mobile_phone', 'office_phone', 'fax_phone', 'email', 'account_code', 'country',
                        'registered_datetime', 'modified_datetime', 'plan_start_date', 'plan_end_date', 'currency_code', 
                        'published_program_configuration', 'published_voucher_configuration', 'membership_configuration', 
-                       'tier_membership_configuration']
+                       'tier_membership_configuration', 'prepaid_configuration']
     
     @staticmethod
     def format_account_code(account_code):
@@ -109,6 +115,14 @@ class MerchantAcct(MerchantMin):
         
         return program_list
     
+    def update_api_key(self):
+        api_key = random_string(24)
+        logger.debug('api_key=%s', api_key)
+        self.api_key = api_key
+        self.put()
+        
+        return api_key
+    
     def flush_dirty_program_configuration(self):
         if self.published_program_configuration and len(self.published_program_configuration)>0:
             existing_programs_list  = self.published_program_configuration.get('programs')
@@ -249,7 +263,6 @@ class MerchantAcct(MerchantMin):
                                                 } 
                                             
         else:
-            self.flush_dirty_program_configuration()
             existing_vouchers_list  = self.published_voucher_configuration.get('vouchers')
             
             voucher_key = voucher_configuration.get('voucher_key')
@@ -268,6 +281,32 @@ class MerchantAcct(MerchantMin):
             
         self.put() 
         
+    def update_prepaid_program(self, prepaid_configuration):
+        if self.prepaid_configuration is None or len(self.prepaid_configuration)==0:
+            self.prepaid_configuration = {
+                                                'programs'  :[prepaid_configuration],
+                                                'count'     : 1,
+                                                } 
+                                            
+        else:
+            existing_prepaid_program_list  = self.prepaid_configuration.get('programs')
+            
+            prepaid_program_key = prepaid_configuration.get('program_key')
+            
+            index = 0
+            for v in existing_prepaid_program_list:
+                if v.get('program_key') == prepaid_program_key:
+                    existing_prepaid_program_list.pop(index)
+                
+                index = index+1
+            
+            existing_prepaid_program_list.append(prepaid_configuration)
+            
+            self.prepaid_configuration['programs']        = existing_prepaid_program_list
+            self.prepaid_configuration['count']           = len(existing_prepaid_program_list) 
+            
+        self.put()     
+        
     def add_membership(self, membership_configuration):
         if is_empty(self.membership_configuration):
             self.membership_configuration = {
@@ -342,19 +381,39 @@ class MerchantAcct(MerchantMin):
             
         self.put()                
         
-    def remove_archieve_program(self, archieve_program_key):
-        self.flush_dirty_program_configuration()
-        existing_programs_list = self.published_program_configuration['programs']
+    def remove_program_from_published_program_configuration(self, program_key_to_remove):
+        
+        logger.debug('remove_program_from_published_program_configuration: program_key_to_remove=%s', program_key_to_remove)
+        
+        #self.flush_dirty_program_configuration()
+        existing_programs_list  = self.published_program_configuration['programs']
+        program_count           = len(existing_programs_list)
+        
+        logger.debug('program_count before remove=%s', program_count)
         
         index = 0
         
         for program in existing_programs_list:
-            if program.get('program_key') == archieve_program_key:
+            
+            logger.debug('program_key=%s', program.get('program_key'))
+            
+            is_same_program_key = program.get('program_key') == program_key_to_remove
+            
+            logger.debug('is_same_program_key=%s', is_same_program_key)
+            
+            if is_same_program_key:
                 existing_programs_list.pop(index)
+                
+                logger.debug('Found program to be remove')
+                
             index = index+1
         
+        program_count = len(existing_programs_list)
+        
+        logger.debug('program_count after remove=%s', program_count)
+        
         self.published_program_configuration['programs']    = existing_programs_list
-        self.published_program_configuration['count']       = len(existing_programs_list)
+        self.published_program_configuration['count']       = program_count
             
         self.put() 
         
@@ -404,7 +463,42 @@ class MerchantAcct(MerchantMin):
         self.tier_membership_configuration['memberships']  = existing_memberships_list
         self.tier_membership_configuration['count']             = len(existing_memberships_list)
             
-        self.put()                
+        self.put()   
+        
+    def remove_prepaid_program_configuration(self, program_key_to_remove):
+        
+        logger.debug('remove_prepaid_program_configuration: program_key_to_remove=%s', program_key_to_remove)
+        if self.prepaid_configuration and self.prepaid_configuration.get('programs'):
+            existing_programs_list  = self.prepaid_configuration['programs']
+            program_count           = len(existing_programs_list)
+            
+            logger.debug('program_count before remove=%s', program_count)
+            
+            index = 0
+            
+            for program in existing_programs_list:
+                
+                logger.debug('program_key=%s', program.get('program_key'))
+                
+                is_same_program_key = program.get('program_key') == program_key_to_remove
+                
+                logger.debug('is_same_program_key=%s', is_same_program_key)
+                
+                if is_same_program_key:
+                    existing_programs_list.pop(index)
+                    
+                    logger.debug('Found program to be remove')
+                    
+                index = index+1
+            
+            program_count = len(existing_programs_list)
+            
+            logger.debug('program_count after remove=%s', program_count)
+            
+            self.prepaid_configuration['programs']    = existing_programs_list
+            self.prepaid_configuration['count']       = program_count
+                
+            self.put()                  
     
     def update_stat_details(self, stat_dict):
         dashboard_stat_figure = self.dashboard_stat_figure
@@ -471,6 +565,10 @@ class MerchantAcct(MerchantMin):
     @staticmethod
     def get_by_account_code(account_code):
         return MerchantAcct.query(ndb.AND(MerchantAcct.account_code==account_code)).get()
+    
+    @staticmethod
+    def get_by_api_key(api_key):
+        return MerchantAcct.query(ndb.AND(MerchantAcct.api_key==api_key)).get()
         
     
     @staticmethod
@@ -497,7 +595,101 @@ class MerchantSentEmail(SentEmail):
     '''
     pass
 
-class MerchantUser(UserMin):
+class Outlet(BaseNModel, DictModel, FullTextSearchable):
+    '''
+    Merchant account as Ancestor
+    '''
+    
+    merchant_acct           = ndb.KeyProperty(name="merchant_acct", kind=MerchantAcct)
+    name                    = ndb.StringProperty(required=True)
+    address                 = ndb.StringProperty(required=False)
+    office_phone            = ndb.StringProperty(required=False)
+    fax_phone               = ndb.StringProperty(required=False)
+    email                   = ndb.StringProperty(required=False)
+    business_hour           = ndb.StringProperty(required=False)
+    is_physical_store       = ndb.BooleanProperty(required=False, default=True)
+    geo_location            = ndb.GeoPtProperty(required=False)
+    created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    
+    fulltextsearch_field_name   = 'name'
+    
+    dict_properties         = ['key', 'name', 'address', 'office_phone', 
+                                'fax_phone', 'email', 'business_hour', 'is_physical_store', 
+                                'geo_location', 'created_datetime']
+    
+    @property
+    def merchant_acct_entity(self):
+        return MerchantAcct.fetch(self.key.parent().urlsafe())
+    
+    @property
+    def merchant_acct_key(self):
+        return self.key.parent().urlsafe().decode('utf-8')
+    
+    
+    @property
+    def outlet_key(self):
+        return self.key.urlsafe()
+    
+    @staticmethod
+    def create(merchant_acct=None,name=None, address=None, email=None, fax_phone=None, 
+               office_phone=None, business_hour=None, geo_location=None, is_physical_store=True):
+        
+        outlet   = Outlet(
+                            parent              = merchant_acct.create_ndb_key(),
+                            name                = name, 
+                            address             = address,
+                            email               = email,
+                            fax_phone           = fax_phone,
+                            office_phone        = office_phone,
+                            business_hour       = business_hour,
+                            geo_location        = geo_location,
+                            is_physical_store   = is_physical_store,
+                            )
+        
+        outlet.put()
+        
+        return outlet
+    
+    @staticmethod
+    def list_by_merchant_acct(merchant_acct):
+        return Outlet.query(ancestor = merchant_acct.create_ndb_key()).fetch(limit=model_conf.MAX_FETCH_RECORD)
+    
+    @staticmethod
+    def list_all_by_merchant_account(merchant_acct, offset=None, start_cursor=None, return_with_cursor=False, keys_only=False, limit = model_conf.MAX_FETCH_RECORD):
+        #condition_query =  Outlet.query(ancestor = merchant_acct.create_ndb_key()).order(-Outlet.created_datetime)
+        condition_query =  Outlet.query(ancestor = merchant_acct.create_ndb_key())
+        return Outlet.list_all_with_condition_query(
+                                        condition_query, 
+                                        offset=offset, 
+                                        start_cursor=start_cursor, 
+                                        return_with_cursor=return_with_cursor, 
+                                        keys_only=keys_only, 
+                                        limit=limit)
+    
+    @staticmethod
+    def count_by_merchant_account(merchant_acct):
+        condition_query = Outlet.query(ancestor = merchant_acct.create_ndb_key())
+        return Outlet.count_with_condition_query(condition_query, limit=model_conf.MAX_FETCH_RECORD)
+    
+    @staticmethod
+    def search_by_merchant_account(name=None, 
+                                 offset=0, start_cursor=None, limit=model_conf.MAX_FETCH_RECORD):
+        
+        search_text_list = None
+        query = Outlet.query()
+        
+        if is_not_empty(name):
+            search_text_list = name.split(' ')
+            
+        total_count                         = Outlet.full_text_count(search_text_list, query, conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH)
+        
+        (search_results, next_cursor)       = Outlet.full_text_search(search_text_list, query, offset=offset, 
+                                                                   start_cursor=start_cursor, return_with_cursor=True, 
+                                                                   limit=limit)
+        
+        return (search_results, total_count, next_cursor)
+
+class MerchantUser(UserMin, FullTextSearchable):
     
     '''
     parent is MerchantAcct
@@ -505,10 +697,13 @@ class MerchantUser(UserMin):
     username                = ndb.StringProperty(required=True)
     permission              = ndb.JsonProperty()
     is_admin                = ndb.BooleanProperty(required=True, default=False)
+    basic_auth_token        = ndb.StringProperty(required=False)
     
-    dict_properties         = ['key', 'user_id', 'name', 'username', 'permission', 
-                                'created_datetime', 'active', 'is_admin', 'permission', 
-                                'is_super_user', 'is_admin_user', 'is_merchant_user']
+    dict_properties         = ['user_id', 'name', 'username', 'permission', 'granted_outlet', 'granted_access',
+                                'created_datetime', 'active', 'is_admin', 'basic_auth_token', 
+                                'is_super_user', 'is_admin_user', 'is_merchant_user', 'merchant_acct_key']
+    
+    fulltextsearch_field_name   = 'name'
     
     @property
     def is_super_user(self):
@@ -526,13 +721,64 @@ class MerchantUser(UserMin):
     def merchant_acct(self):
         return MerchantAcct.fetch(self.key.parent().urlsafe())
     
+    @property
+    def merchant_acct_key(self):
+        return self.key.parent().urlsafe().decode('utf-8')
+    
     @property
     def granted_outlet(self):
-        return self.permission.get('granted_outlet')
+        if self.is_admin:
+            logger.debug('is admin merchant user')
+            g_outlets_list = []
+            all_outlet_list = Outlet.list_by_merchant_acct(self.merchant_acct)
+            
+            logger.debug('all_outlet_list=%s', all_outlet_list)
+            
+            for o in all_outlet_list:
+                g_outlets_list.append(o.key_in_str)
+                
+            return g_outlets_list
+            
+        else:
+            if self.permission:
+                return self.permission.get('granted_outlet')
+            else:
+                return []
+        
+    @property
+    def granted_outlet_details_list(self):
+        g_outlets_list = []
+        
+        if self.is_admin:
+            logger.debug('is admin merchant user')
+            
+            all_outlet_list = Outlet.list_by_merchant_acct(self.merchant_acct)
+            
+            for o in all_outlet_list:
+                g_outlets_list.append({
+                                        'outlet_key'    : o.key_in_str,
+                                        'name'          : o.name,
+                                        })
+                
+            return g_outlets_list
+            
+        else:
+            outlet_key_list =  self.permission.get('granted_outlet')
+            
+            for o in  outlet_key_list:
+                outlet_details = Outlet.fetch(o)
+                g_outlets_list.append({
+                                        'outlet-key'    : 0,
+                                        'name'          : outlet_details.name,
+                                        })
+        return g_outlets_list
     
     @property
     def granted_access(self):
-        return self.permission.get('granted_access')
+        if self.permission:
+            return self.permission.get('granted_access')
+        else:
+            return []
     
     @staticmethod
     def update_permission(merchant_user, access_permission, outlet_permission, is_admin=False):
@@ -598,53 +844,32 @@ class MerchantUser(UserMin):
     
     @staticmethod
     def get_merchant_acct_by_merchant_user(merchant_user):
-        return MerchantAcct.fetch(merchant_user.key.parent().urlsafe())
+        return MerchantUser.fetch(merchant_user.key.parent().urlsafe())
     
-class Outlet(BaseNModel, DictModel, FullTextSearchable):
-    merchant_acct           = ndb.KeyProperty(name="merchant_acct", kind=MerchantAcct)
-    name                    = ndb.StringProperty(required=True)
-    address                 = ndb.StringProperty(required=False)
-    office_phone            = ndb.StringProperty(required=False)
-    fax_phone               = ndb.StringProperty(required=False)
-    email                   = ndb.StringProperty(required=False)
-    business_hour           = ndb.StringProperty(required=False)
-    is_physical_store       = ndb.BooleanProperty(required=False, default=True)
-    geo_location            = ndb.GeoPtProperty(required=False)
-    created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
-    
-    searchable_field_name   = 'name'
-    
-    dict_properties         = ['key', 'name', 'address', 'office_phone', 
-                                'fax_phone', 'email', 'business_hour', 'is_physical_store', 
-                                'geo_location', 'created_datetime']
-    
-    @property
-    def outlet_key(self):
-        return self.key.urlsafe()
     
     @staticmethod
-    def create(merchant_acct=None,name=None, address=None, email=None, fax_phone=None, 
-               office_phone=None, business_hour=None, geo_location=None, is_physical_store=True):
+    def search_by_merchant_account(name=None, username=None,
+                                 offset=0, start_cursor=None, limit=model_conf.MAX_FETCH_RECORD):
         
-        outlet   = Outlet(
-                            parent              = merchant_acct.create_ndb_key(),
-                            name                = name, 
-                            address             = address,
-                            email               = email,
-                            fax_phone           = fax_phone,
-                            office_phone        = office_phone,
-                            business_hour       = business_hour,
-                            geo_location        = geo_location,
-                            is_physical_store   = is_physical_store,
-                            )
+        search_text_list = None
+        query = MerchantUser.query()
         
-        outlet.put()
+        if is_not_empty(name):
+            search_text_list = name.split(' ')
+            
+        elif is_not_empty(username):
+            query = query.filter(MerchantUser.username==username)
+            
         
-        return outlet
+        
+        total_count                         = MerchantUser.full_text_count(search_text_list, query, conf.MAX_FETCH_RECORD_FULL_TEXT_SEARCH)
+        
+        (search_results, next_cursor)       = MerchantUser.full_text_search(search_text_list, query, offset=offset, 
+                                                                   start_cursor=start_cursor, return_with_cursor=True, 
+                                                                   limit=limit)
+        
+        return (search_results, total_count, next_cursor)
     
-    @staticmethod
-    def list_by_merchant_acct(merchant_acct):
-        return Outlet.query(ancestor = merchant_acct.create_ndb_key()).fetch(limit=model_conf.MAX_FETCH_RECORD)
     
 class MerchantTagging(Tagging):    
     
