@@ -14,41 +14,35 @@ import logging
 from trexlib.utils.common.cache_util import cache
 from trexmodel import conf, program_conf
 from datetime import datetime
+from trexmodel.models.datastore.model_decorators import model_transactional
 
 
 logger = logging.getLogger('model')
 
 
 class ProgramBase(BaseNModel, DictModel):
-    reward_base             = ndb.StringProperty(required=True, choices=set(program_conf.REWARD_BASE_SET))
-    reward_format           = ndb.StringProperty(required=False, choices=set(program_conf.REWARD_FORMAT_SET))
-    completed_status        = ndb.StringProperty(required=True, choices=set(program_conf.PROGRAM_STATUS))
-    giveaway_method         = ndb.StringProperty(required=False, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
-    start_date              = ndb.DateProperty(required=True)
-    end_date                = ndb.DateProperty(required=True)
-    desc                    = ndb.StringProperty(required=False)
-    program_settings        = ndb.JsonProperty(required=True)
-    created_datetime        = ndb.DateTimeProperty(required=True, auto_now_add=True)
-    modified_datetime       = ndb.DateTimeProperty(required=True, auto_now=True)
-    published_datetime      = ndb.DateTimeProperty(required=False)
-    archived_datetime       = ndb.DateTimeProperty(required=False)
+    reward_base                         = ndb.StringProperty(required=True, choices=set(program_conf.REWARD_BASE_SET))
+    reward_format                       = ndb.StringProperty(required=False, choices=set(program_conf.REWARD_FORMAT_SET))
+    completed_status                    = ndb.StringProperty(required=True, choices=set(program_conf.PROGRAM_STATUS))
+    giveaway_method                     = ndb.StringProperty(required=False, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
+    start_date                          = ndb.DateProperty(required=True)
+    end_date                            = ndb.DateProperty(required=True)
+    label                               = ndb.StringProperty(required=False)
+    desc                                = ndb.StringProperty(required=False)
+    program_settings                    = ndb.JsonProperty(required=True)
+    created_datetime                    = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    modified_datetime                   = ndb.DateTimeProperty(required=True, auto_now=True)
+    published_datetime                  = ndb.DateTimeProperty(required=False)
+    archived_datetime                   = ndb.DateTimeProperty(required=False)
+    remarks                             = ndb.StringProperty(required=False)
     
+    created_by                          = ndb.KeyProperty(name="created_by", kind=MerchantUser)
+    created_by_username                 = ndb.StringProperty(required=False)
+    modified_by                         = ndb.KeyProperty(name="modified_by", kind=MerchantUser)
+    modified_by_username                = ndb.StringProperty(required=False)
     
-    created_by              = ndb.KeyProperty(name="created_by", kind=MerchantUser)
-    created_by_username     = ndb.StringProperty(required=False)
-    modified_by             = ndb.KeyProperty(name="modified_by", kind=MerchantUser)
-    modified_by_username    = ndb.StringProperty(required=False)
-    
-    enabled                 = ndb.BooleanProperty(default=True)
-    archived                = ndb.BooleanProperty(default=False)
-    
-    dict_properties         = ['reward_base', 'giveaway_method', 'reward_format', 'completed_status', 'start_date', 'end_date', 'desc', 'program_settings', 
-                               'created_datetime', 'modified_datetime',  'enabled','completed_status','is_enabled', 'is_disabled', 'is_review_state', 
-                               'is_published', 'archived', 'is_reward_amount_required', 'completed_prograss_in_percentage', 'completed_status_index',
-                               'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
-                               'is_recurring_scheme', 'is_multitier_scheme', 'is_multitier_membership',
-                               'expiration_type', 'expiration_date', 'expiration_value', 'is_expiration_date_type',
-                               'created_by_username', 'modified_by_username']
+    enabled                             = ndb.BooleanProperty(default=True)
+    archived                            = ndb.BooleanProperty(default=False)
     
     @property
     def is_enabled(self):
@@ -140,23 +134,37 @@ class ProgramBase(BaseNModel, DictModel):
         if self.program_settings.get('exclusivity') and self.program_settings.get('exclusivity').get('tier_memberships'):
             return ','.join(self.program_settings.get('exclusivity').get('tier_memberships')) or ''        
     
+    @property
+    def giveaway_reward_when(self):
+        if self.program_settings.get('scheme') and self.program_settings.get('scheme').get('giveaway_when'):
+            return self.program_settings.get('scheme').get('giveaway_when')
+        
+    @property
+    def giveaway_reward_advance_in_day(self):
+        if self.program_settings.get('scheme') and self.program_settings.get('scheme').get('advance_in_day'):
+            return self.program_settings.get('scheme').get('advance_in_day')
+        else:
+            return 0    
+    
     def to_program_configuration(self):
         program_configuration = {
-                                'program_key'       : self.key_in_str,
-                                'desc'              : self.desc,
-                                'reward_base'       : self.reward_base,
-                                'reward_format'     : self.reward_format,
-                                'giveaway_method'   : self.giveaway_method,
-                                'start_date'        : self.start_date.strftime('%d-%m-%Y'),
-                                'end_date'          : self.end_date.strftime('%d-%m-%Y'),    
-                                'program_settings'  : self.program_settings,
-                                'is_published'      : self.is_published,   
+                                'merchant_acct_key'                 : self.parent_key,
+                                'program_key'                       : self.key_in_str,
+                                'desc'                              : self.desc,
+                                'reward_base'                       : self.reward_base,
+                                'reward_format'                     : self.reward_format,
+                                'giveaway_method'                   : self.giveaway_method,
+                                'start_date'                        : self.start_date.strftime('%d-%m-%Y'),
+                                'end_date'                          : self.end_date.strftime('%d-%m-%Y'),    
+                                'program_settings'                  : self.program_settings,
+                                'is_published'                      : self.is_published,  
+                                'remarks'                           : self.remarks, 
                                 }
         
         return program_configuration
     
     @classmethod
-    def update_program_base_data(cls, program, reward_base=None, reward_format=None, giveaway_method=None, 
+    def update_program_base_data(cls, program, reward_base=None, reward_format=None,  
                                  desc=None,start_date=None, end_date=None, modified_by=None):
         
         modified_by_username = None
@@ -165,41 +173,40 @@ class ProgramBase(BaseNModel, DictModel):
             if isinstance(modified_by, MerchantUser):
                 modified_by_username = modified_by.username
         
+        program_settings = program.program_settings or {}
+        
+        exclusivity_configuration = {
+                                                    'tags'              : [],
+                                                    'memberships'       : [],
+                                                    'tier_memberships'  : [],
+                                                    }
+        
+        program_settings['exclusivity'] = exclusivity_configuration
+        
         program.reward_base            = reward_base
         program.reward_format          = reward_format
-        program.giveaway_method        = giveaway_method
         program.desc                   = desc
         program.start_date             = start_date
         program.end_date               = end_date
         program.modified_by            = modified_by.create_ndb_key()
         program.modified_by_username   = modified_by_username
         program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_BASE
+        program.program_settings       = program_settings
+        
         
         
         program.put()
         
         return program
-    '''
+    
     @classmethod
-    def update_prorgram_reward_format_data(cls, program, reward_format=None, modified_by=None):
-        modified_by_username = None
-        
-        if is_not_empty(modified_by):
-            if isinstance(modified_by, MerchantUser):
-                modified_by_username = modified_by.username
-        
-        program.reward_format          = reward_format
-        program.modified_by            = modified_by.create_ndb_key()
-        program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_FORMAT
-        
-        
-        program.put()
-        
-        return program
-    '''
-    @classmethod
-    def update_prorgram_reward_scheme_data(cls, program, reward_scheme_configuration={}, modified_by=None):
+    def update_prorgram_reward_details_data(cls, program, 
+                                            giveaway_method=None,
+                                            giveaway_system_condition = None, 
+                                            giveaway_system_condition_membership_key=None, 
+                                            giveaway_system_condition_tier_membership_key=None,
+                                            reward_scheme_configuration={}, modified_by=None,
+                                            remarks=None):
         modified_by_username = None
         
         if is_not_empty(modified_by):
@@ -214,12 +221,39 @@ class ProgramBase(BaseNModel, DictModel):
         else:
             program_settings = program.program_settings
             program_settings['scheme']              = reward_scheme_configuration
+            
+        if program_conf.REWARD_BASE_ON_GIVEAWAY == program.reward_base:  
+            program.giveaway_method     = giveaway_method
+            giveaway_system_settings    = program_settings.get('giveaway_system_settings')
+            
+            if giveaway_system_settings is None:        
+                giveaway_system_settings = {
+                                                    'giveaway_system_condition': giveaway_system_condition
+                                                }
+            else:
+                giveaway_system_settings['giveaway_system_condition'] = giveaway_system_condition
                 
-        program.modified_by            = modified_by.create_ndb_key()
-        program.modified_by_username   = modified_by_username
-        program.completed_status       = program_conf.PROGRAM_STATUS_REWARD_SCHEME
-        program.program_settings       = program_settings
+            if is_not_empty(giveaway_system_condition_membership_key):
+                giveaway_system_settings['giveaway_memberships'] = giveaway_system_condition_membership_key
+            else:
+                if giveaway_system_settings.get('giveaway_memberships'):
+                    del giveaway_system_settings['giveaway_memberships']
+            
+            if is_not_empty(giveaway_system_condition_tier_membership_key):
+                giveaway_system_settings['giveaway_tier_memberships'] = giveaway_system_condition_tier_membership_key
+            else:
+                if giveaway_system_settings.get('giveaway_tier_memberships'):
+                    del giveaway_system_settings['giveaway_tier_memberships']
+            
+            logger.debug('######################## giveaway_system_settings=%s', giveaway_system_settings)
         
+            program_settings['giveaway_system_settings'] = giveaway_system_settings    
+                
+        program.modified_by             = modified_by.create_ndb_key()
+        program.modified_by_username    = modified_by_username
+        program.completed_status        = program_conf.PROGRAM_STATUS_REWARD_SCHEME
+        program.program_settings        = program_settings
+        program.remarks                 = remarks
         
         program.put()
         
@@ -253,7 +287,6 @@ class ProgramBase(BaseNModel, DictModel):
         program.modified_by_username   = modified_by_username
         program.program_settings       = program_settings
         
-        
         program.put()
         
         return program
@@ -422,60 +455,131 @@ class ProgramBase(BaseNModel, DictModel):
         program.modified_by                     = modified_by.create_ndb_key()
         program.modified_by_username            = modified_by_username
         program.put()    
+    
+    @classmethod
+    def __create_schedule_program(cls, merchant_acct, program):
+        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
+            if program.reward_base == program_conf.REWARD_BASE_ON_BIRTHDAY:
+                if program.giveaway_reward_when == program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START:
+                    MerchantScheduleProgram.create_first_day_of_month_schedule_program(merchant_acct, program)
+                elif program.giveaway_reward_when == program_conf.ADVANCE_IN_DAY:
+                    MerchantScheduleProgram.create_daily_schedule_program(merchant_acct, program)
+    
+    @classmethod
+    def __remove_schedule_program(cls, program):
+        if program.reward_base in (program_conf.SCHEDULE_BASED_PROGRAM):
+            MerchantScheduleProgram.remove_by_merchant_program(program)
         
     @classmethod
+    @model_transactional(desc="publish_program")
     def publish_program(cls, program):
         program.completed_status = program_conf.PROGRAM_STATUS_PUBLISH
         program.published_datetime = datetime.now()
         program.put()
         
-        
         merchant_acct = program.merchant_acct
         merchant_acct.update_published_program(program.to_program_configuration())
         
+        cls.__create_schedule_program(merchant_acct, program)
+        
     @classmethod
+    @model_transactional(desc="archive_program")
     def archive_program(cls, program):
         program.archived = True
         program.archived_datetime = datetime.now()
         program.put()
         
         merchant_acct = program.merchant_acct
-        merchant_acct.remove_archieve_program(program.key_in_str)    
+        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str) 
+        
+        cls.__remove_schedule_program(program)
+        
     
     @classmethod
+    @model_transactional(desc="enable program")
     def enable(cls, program):
         program.enabled = True
         program.put()
         
+        if program.is_published:
+            merchant_acct = program.merchant_acct
+            merchant_acct.update_published_program(program.to_program_configuration())
+            
+            cls.__create_schedule_program(merchant_acct, program)
+            
+        
     @classmethod
+    @model_transactional(desc="disable program")
     def disable(cls, program):
         program.enabled = False
         program.put()  
+        
+        merchant_acct = program.merchant_acct
+        merchant_acct.remove_program_from_published_program_configuration(program.key_in_str)
+        
+        cls.__remove_schedule_program(program)
     
 class MerchantProgram(ProgramBase):
     #merchant_acct           = ndb.KeyProperty(name="merchant_acct", kind=MerchantAcct) 
     
     
+    dict_properties                     = ['reward_base', 'giveaway_method', 'reward_format', 'completed_status', 'start_date', 'end_date', 'desc', 'program_settings', 
+                                           'created_datetime', 'modified_datetime',  'enabled','completed_status','is_enabled', 'is_disabled', 'is_review_state', 
+                                           'is_published', 'archived', 'is_reward_amount_required', 'completed_prograss_in_percentage', 'completed_status_index',
+                                           'exclusive_tags_list', 'exclusive_memberships_list', 'exclusive_tier_memberships_list',
+                                           'is_recurring_scheme', 'is_multitier_scheme', 'is_multitier_membership',
+                                           'expiration_type', 'expiration_date', 'expiration_value', 'is_expiration_date_type',
+                                           'giveaway_reward_when', 'giveaway_reward_advance_in_day',
+                                           'giveaway_system_condition', 'giveaway_system_condition_memberships_list', 'giveaway_system_condition_tier_memberships_list',
+                                           'created_by_username', 'modified_by_username']
+    
+    @property
+    def giveaway_system_condition(self):
+        if self.program_settings and self.program_settings.get('giveaway_system_settings'):
+            return self.program_settings.get('giveaway_system_settings').get('giveaway_system_condition')
+    
+    @property
+    def giveaway_system_condition_memberships_list(self):
+        if self.program_settings and self.program_settings.get('giveaway_system_settings'):
+            return self.program_settings.get('giveaway_system_settings').get('giveaway_memberships')
+    
+    @property
+    def giveaway_system_condition_tier_memberships_list(self):
+        if self.program_settings and self.program_settings.get('giveaway_system_settings'):
+            return self.program_settings.get('giveaway_system_settings').get('giveaway_tier_memberships')
+    
     @staticmethod
-    def create(merchant_acct, reward_base=None, giveaway_method=None, reward_format=None, desc=None, start_date=None, end_date=None, created_by=None):
+    def create(merchant_acct, reward_base=None, reward_format=None, 
+               desc=None, start_date=None, end_date=None, created_by=None):
+        
         created_by_username = None
         if is_not_empty(created_by):
             if isinstance(created_by, MerchantUser):
                 created_by_username = created_by.username
         
+        exclusivity_configuration = {
+                                                    'tags'              : [],
+                                                    'memberships'       : [],
+                                                    'tier_memberships'  : [],
+                                                    }
+        ''
+        program_settings = {
+                            'exclusivity'               : exclusivity_configuration
+                            }
+        
+        
         
         merchant_program =  MerchantProgram(
-                                        parent              = merchant_acct.create_ndb_key(),
-                                        reward_base         = reward_base,
-                                        giveaway_method     = giveaway_method,
-                                        reward_format       = reward_format,
-                                        desc                = desc,
-                                        start_date          = start_date,
-                                        end_date            = end_date,
-                                        created_by          = created_by.create_ndb_key(),
-                                        created_by_username = created_by_username,
-                                        completed_status    = program_conf.PROGRAM_STATUS_REWARD_BASE,
-                                        program_settings    = {},
+                                        parent                              = merchant_acct.create_ndb_key(),
+                                        reward_base                         = reward_base,
+                                        reward_format                       = reward_format,
+                                        desc                                = desc,
+                                        start_date                          = start_date,
+                                        end_date                            = end_date,
+                                        created_by                          = created_by.create_ndb_key(),
+                                        created_by_username                 = created_by_username,
+                                        completed_status                    = program_conf.PROGRAM_STATUS_REWARD_BASE,
+                                        program_settings                    = program_settings,
                                         )
         
         merchant_program.put()
@@ -497,9 +601,102 @@ class MerchantProgram(ProgramBase):
     def list_archived_by_merchant_account(merchant_acct):
         return MerchantProgram.query(ndb.AND(MerchantProgram.archived==True), ancestor=merchant_acct.create_ndb_key()).fetch(limit=model_conf.MAX_FETCH_RECORD)
 
-class BrandProgram(ProgramBase): 
-    #corporate_acct           = ndb.KeyProperty(name="corporate_acct", kind=CorporateAcct)
-    pass
+class MerchantScheduleProgram(BaseNModel, DictModel):
+    '''
+    Merchant acct as ancestor. Used for schedule program like birthday or push reward.
+    '''
+    merchant_program                    = ndb.KeyProperty(name="merchant_program", kind=MerchantProgram)
+    program_configuration               = ndb.JsonProperty(required=True)
+    schedule_type                       = ndb.StringProperty(required=True, choices=set(program_conf.PROGRAM_SCHEDULE_TYPE_GROUP))
+    start_date                          = ndb.DateProperty(required=True)
+    end_date                            = ndb.DateProperty(required=True)
+    created_datetime                    = ndb.DateTimeProperty(required=True, auto_now_add=True)
+    
+    dict_properties                     = ['program_configuration', 'schedule_type', 'start_date', 'end_date']
+    
+    
+    @property
+    def merchant_acct_key(self):
+        return self.parent_key
+    
+    @property
+    def merchant_program_entity(self):
+        return MerchantProgram.fetch(self.merchant_program.urlsafe())
+    
+    @property
+    def merchant_program_desc(self):
+        return self.merchant_program_entity.desc
+    
+    @staticmethod
+    def __list_by_schedule_type(schedule_type):
+        return MerchantScheduleProgram.query(MerchantScheduleProgram.schedule_type==schedule_type).fetch(limit=model_conf.MAX_FETCH_RECORD)
+    
+    @staticmethod
+    def list_daily_schedule_program():
+        return MerchantScheduleProgram.__list_by_schedule_type(program_conf.PROGRAM_SCHEDULE_TYPE_DAILY)
+    
+    @staticmethod
+    def list_beginning_of_month_schedule_program():
+        return MerchantScheduleProgram.__list_by_schedule_type(program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START)
+    
+    @staticmethod
+    def list_beginning_of_week_schedule_program():
+        return MerchantScheduleProgram.__list_by_schedule_type(program_conf.PROGRAM_SCHEDULE_TYPE_MONDAY)
+    
+    @staticmethod
+    def list_friday_schedule_program():
+        return MerchantScheduleProgram.__list_by_schedule_type(program_conf.PROGRAM_SCHEDULE_TYPE_FRIDAY)
+    
+    @staticmethod
+    def get_by_merchant_program(merchant_program):
+        return MerchantScheduleProgram.query(MerchantScheduleProgram.merchant_program == merchant_program.create_ndb_key()).get()
+    
+    @staticmethod
+    def __create(merchant_acct, schedule_type, merchant_program):
+        schedule_program    = MerchantScheduleProgram(
+                                parent                  = merchant_acct.create_ndb_key(),
+                                merchant_program        = merchant_program.create_ndb_key(),
+                                schedule_type           = schedule_type,
+                                program_configuration   = merchant_program.to_program_configuration(),
+                                start_date              = merchant_program.start_date,
+                                end_date                = merchant_program.end_date,
+                                
+                            )
+        
+        schedule_program.put()
+        
+        return schedule_program
+        
+    @staticmethod
+    def create_daily_schedule_program(merchant_acct, merchant_program):
+        return MerchantScheduleProgram.__create(merchant_acct, program_conf.PROGRAM_SCHEDULE_TYPE_DAILY, merchant_program)
+    
+    @staticmethod
+    def create_first_day_of_month_schedule_program(merchant_acct, merchant_program):
+        return MerchantScheduleProgram.__create(merchant_acct, program_conf.PROGRAM_SCHEDULE_TYPE_MONTH_START, merchant_program)
+    
+    @staticmethod
+    def create_every_monday_schedule_program(merchant_acct, merchant_program):
+        return MerchantScheduleProgram.__create(merchant_acct, program_conf.PROGRAM_SCHEDULE_TYPE_MONDAY, merchant_program)
+    
+    @staticmethod
+    def create_every_friday_schedule_program(merchant_acct, merchant_program):
+        return MerchantScheduleProgram.__create(merchant_acct, program_conf.PROGRAM_SCHEDULE_TYPE_FRIDAY, merchant_program)
+    
+    @staticmethod
+    def create_weekend_schedule_program(merchant_acct, merchant_program):
+        return MerchantScheduleProgram.__create(merchant_acct, program_conf.PROGRAM_SCHEDULE_TYPE_WEEKEND, merchant_program)    
+        
+    @staticmethod
+    def remove_by_merchant_program(merchant_program):
+        schedule_program = MerchantScheduleProgram.get_by_merchant_program(merchant_program)
+        if schedule_program:
+            logger.debug('Found schedule program')
+            schedule_program.delete()
+        else:
+            logger.warn('Schedule program not found')
+        
+    
 
 
            
