@@ -4,7 +4,7 @@ Created on 22 Apr 2021
 @author: jacklok
 '''
 from google.cloud import ndb
-from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel, FullTextSearchable
+from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
 from trexmodel.models.datastore.user_models import User
 from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet, MerchantUser
 from trexmodel.models.datastore.program_models import MerchantProgram
@@ -16,7 +16,7 @@ from trexlib.utils.string_util import random_string
 from datetime import datetime, timedelta
 from trexmodel.utils.model.model_util import generate_transaction_id,\
     string_to_key_property
-
+from six import string_types
 
 logger = logging.getLogger('model')
 
@@ -26,6 +26,7 @@ class RewardEntitlement(BaseNModel, DictModel):
     expiry_date                 = ndb.DateProperty(required=True)
     
     transaction_id              = ndb.StringProperty(required=True)
+    
     invoice_id                  = ndb.StringProperty(required=False)
     
     rewarded_datetime           = ndb.DateTimeProperty(required=True, auto_now_add=True)
@@ -36,7 +37,7 @@ class RewardEntitlement(BaseNModel, DictModel):
     
     status                      = ndb.StringProperty(required=False, default=program_conf.REWARD_STATUS_VALID)
     
-    reverted_datetime           = ndb.DateTimeProperty(required=True)
+    reverted_datetime           = ndb.DateTimeProperty(required=False)
     reverted_by                 = ndb.KeyProperty(name="reverted_by", kind=MerchantUser)
     reverted_by_username        = ndb.StringProperty(required=False)
     
@@ -62,6 +63,10 @@ class CustomerEntitledReward(RewardEntitlement):
     transact_outlet             = ndb.KeyProperty(name="transact_outlet", kind=Outlet)
     reward_program              = ndb.KeyProperty(name="reward_program", kind=MerchantProgram)
     
+    @property
+    def entitled_customer_key(self):
+        return self.key.parent().urlsafe().decode('utf-8')
+    
     @property
     def rewarded_by_merchant_acct_entity(self):
         return MerchantAcct.fetch(self.merchant_acct.urlsafe())
@@ -101,17 +106,24 @@ class CustomerEntitledReward(RewardEntitlement):
     @classmethod
     def list_by_customer(cls, customer, status=program_conf.REWARD_STATUS_VALID, limit = conf.MAX_FETCH_RECORD):
         return cls.query(ndb.AND(cls.status==status), ancestor=customer.create_ndb_key()).fetch(limit=limit)
+        
+    @classmethod
+    def list_all_by_customer(cls, customer, limit = conf.MAX_FETCH_RECORD):
+        return cls.query(ancestor=customer.create_ndb_key()).fetch(limit=limit)    
 
 class CustomerCountableReward(CustomerEntitledReward):
-    reward_amount               = ndb.IntegerProperty(required=True, default=0)
-    used_reward_amount          = ndb.IntegerProperty(required=True, default=0)
+    reward_amount               = ndb.FloatProperty(required=True, default=0)
+    used_reward_amount          = ndb.FloatProperty(required=True, default=0)
     
     @property
     def reward_balance(self):
         return self.reward_amount - self.used_reward_amount
     
     def update_used_reward_amount(self, used_reward_amount):
-        self.used_reward_amount = used_reward_amount
+        self.used_reward_amount += used_reward_amount
+        
+        logger.debug('CustomerCountableReward: reward_balance=%s', self.reward_balance)
+        
         if self.reward_balance ==0:
             self.status = program_conf.REWARD_STATUS_REDEEMED
         
@@ -120,7 +132,7 @@ class CustomerCountableReward(CustomerEntitledReward):
     @classmethod
     def list_by_valid_with_cursor(cls, customer, limit=50, start_cursor=None):
         query = cls.query(ndb.AND(cls.status==program_conf.REWARD_STATUS_VALID
-            ), ancestor=customer.create_ndb_key()).order(-cls.expiry_date)
+            ), ancestor=customer.create_ndb_key()).order(cls.expiry_date)
             
         (result, next_cursor) = cls.list_all_with_condition_query(query, start_cursor=start_cursor, return_with_cursor=True, limit=limit)
         
@@ -137,8 +149,8 @@ class CustomerCountableReward(CustomerEntitledReward):
     def to_reward_summary(self):
         return {
                 'reward_format'         : self.reward_format, 
-                'reward_amount'         : self.reward_amount,
-                'used_reward_amount'    : self.used_reward_amount,
+                'amount'                : self.reward_amount,
+                'used_amount'           : self.used_reward_amount,
                 'expiry_date'           : self.expiry_date.strftime('%d-%m-%Y'),
                 }
     
@@ -166,13 +178,13 @@ class CustomerCountableReward(CustomerEntitledReward):
                                             invoice_id              = invoice_id,
                                             
                                             merchant_acct           = customer_acct.merchant_acct,
-                                            transact_outlet         = transact_outlet.create_ndb_key(),
+                                            transact_outlet         = transact_outlet.create_ndb_key() if transact_outlet else None,
                                             
                                             effective_date          = effective_date,
                                             expiry_date             = expiry_date,
                                             
-                                            rewarded_by             = rewarded_by.create_ndb_key(),
-                                            rewarded_by_username    = rewarded_by.username,
+                                            rewarded_by             = rewarded_by.create_ndb_key() if rewarded_by else None,
+                                            rewarded_by_username    = rewarded_by.username if rewarded_by else None,
                                             
                                             reward_program          = reward_program,
                                             rewarded_datetime       = rewarded_datetime,
@@ -271,8 +283,8 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         self.redeemed_datetime      = redeemed_datetime
         self.redeemed_by            = redeemed_by.create_ndb_key()
         self.redeemed_by_username   = redeemed_by.username
-        self.put() 
-    
+        self.put()
+        
     @property
     def is_used(self):
         return self.status == program_conf.REWARD_STATUS_REDEEMED
@@ -299,6 +311,21 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
     def entitled_voucher_entity(self):
         return MerchantVoucher.fetch(self.entitled_voucher.urlsafe())
     
+    @property
+    def merchant_voucher(self):
+        return MerchantVoucher.fetch(self.entitled_voucher.urlsafe())
+    
+    @property
+    def merchant_voucher_key(self):
+        return self.entitled_voucher.urlsafe().decode('utf-8')
+    
+    def to_redeem_info(self):
+        return {
+                'redeem_code'       : self.redeem_code,
+                'effective_date'    : self.effective_date.strftime('%d-%m-%Y'),
+                'expiry_date'       : self.expiry_date.strftime('%d-%m-%Y'),
+                }
+    
     @staticmethod
     def create(entitled_voucher, customer_acct, transact_outlet, 
                effective_date=None, expiry_date=None,
@@ -322,10 +349,10 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
                                             invoice_id              = invoice_id,
                                             
                                             merchant_acct           = customer_acct.merchant_acct,
-                                            transact_outlet         = transact_outlet.create_ndb_key(),
+                                            transact_outlet         = transact_outlet.create_ndb_key() if transact_outlet else None,
                                             
-                                            rewarded_by             = rewarded_by.create_ndb_key(),
-                                            rewarded_by_username    = rewarded_by.username,
+                                            rewarded_by             = rewarded_by.create_ndb_key() if rewarded_by else None,
+                                            rewarded_by_username    = rewarded_by.username if rewarded_by else None,
                                             
                                             rewarded_datetime       = rewarded_datetime,
                                             
@@ -333,6 +360,8 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         
         customer_entiteld_voucher.put()
         
+        return customer_entiteld_voucher
+        
         
     @staticmethod
     def get_by_redeem_code(redeem_code):
@@ -341,8 +370,13 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
     @staticmethod
     def list_by_customer(customer, limit=conf.MAX_FETCH_RECORD, offset=0, voucher_status=program_conf.REWARD_STATUS_VALID):
         return CustomerEntitledVoucher.query(ndb.AND(CustomerEntitledVoucher.status==voucher_status),
-            ancestor=customer.create_ndb_key()).fetch(limit=limit, offset=offset)
+                ancestor=customer.create_ndb_key()).fetch(limit=limit, offset=offset)
+
+    @staticmethod
+    def list_all_by_customer(customer, limit=conf.MAX_FETCH_RECORD, offset=0):
+        return CustomerEntitledVoucher.query(ancestor=customer.create_ndb_key()).fetch(limit=limit, offset=offset)
     
+        
     @staticmethod
     def update_customer_entiteld_voucher_summary(customer, voucher_status=program_conf.REWARD_STATUS_VALID):
         customers_vouchers_list = CustomerEntitledVoucher.list_by_customer(customer, voucher_status=voucher_status)
@@ -362,13 +396,18 @@ class CustomerEntitledVoucher(CustomerEntitledReward):
         customer.entitled_voucher_summary = entitled_voucher_summary
         customer.put()
     
-class VoucherRewardDetails(object):    
-    
+class VoucherRewardDetailsForUpstreamData(object):    
+    '''
+    For Upstream purpose
+    '''
     def __init__(self, voucher_key, reward_amount, expiry_date, rewarded_datetime, merchant_acct=None):
         self.merchant_acct      = merchant_acct
         self.voucher_key        = voucher_key
         self.reward_amount      = reward_amount
-        self.expiry_date        = expiry_date
+        if isinstance(expiry_date, string_types):
+            self.expiry_date        = datetime.strptime(expiry_date, '%d-%m-%Y').date()
+        else:
+            self.expiry_date = expiry_date
         self.rewarded_datetime  = rewarded_datetime
         
     def __repr__(self, *args, **kwargs):
