@@ -15,13 +15,13 @@ from trexmodel.models.datastore.merchant_models import MerchantAcct, Outlet,\
 import logging
 from trexmodel import conf, program_conf
 from datetime import datetime,date, timedelta
-from trexmodel.conf import HANDLE_DATETIME_WITH_GMT, SERVER_DATETIME_GMT
 from trexmodel.models.datastore.reward_models import CustomerPointReward,\
     CustomerStampReward, CustomerEntitledVoucher
 from trexmodel.models.datastore.customer_model_helpers import update_reward_summary_with_reverted_reward,\
-    update_customer_entiteld_voucher_summary_after_reverted
+    update_customer_entiteld_voucher_summary_after_reverted_voucher
 from trexmodel.utils.model.model_util import generate_transaction_id
 from gettext import gettext
+from trexmodel.models.datastore.prepaid_models import CustomerPrepaidReward
 
 logger = logging.getLogger('model')
 
@@ -46,21 +46,30 @@ class CustomerTransaction(BaseNModel, DictModel):
     transact_by                 = ndb.KeyProperty(name="created_by", kind=MerchantUser)
     transact_by_username        = ndb.StringProperty(required=False)
     
+    sales_channel               = ndb.StringProperty(required=False)
+    
     entitled_reward_summary     = ndb.JsonProperty()
     entitled_voucher_summary    = ndb.JsonProperty()
+    entitled_prepaid_summary    = ndb.JsonProperty()
     
     reward_giveaway_method      = ndb.StringProperty(required=False, default=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM)
     
+    allow_to_revert             = ndb.BooleanProperty(required=False, default=True)
+    
     is_revert                   = ndb.BooleanProperty(required=False, default=False)
     reverted_datetime           = ndb.DateTimeProperty(required=False)
     reverted_by                 = ndb.KeyProperty(name="reverted_by", kind=MerchantUser)
     reverted_by_username        = ndb.StringProperty(required=False)
     
+    is_reward_redeemed          = ndb.BooleanProperty(required=False, default=False)
+    
+    is_sales_transaction        = ndb.BooleanProperty(required=False, default=True)
     
     dict_properties         = ['transaction_id', 'invoice_id', 'remarks', 'tax_amount', 'transact_amount', 'reward_giveaway_method',
-                               'entitled_reward_summary', 'entitled_voucher_summary', 'transact_customer_acct', 'transact_outlet_details', 'transact_merchant_acct',
+                               'entitled_reward_summary', 'entitled_voucher_summary', 'entitled_prepaid_summary', 
+                               'transact_customer_acct', 'transact_outlet_details', 'transact_merchant_acct',
                                'transact_datetime', 'created_datetime',  'transact_outlet_key', 'is_revert', 'reverted_datetime',
-                               'transact_by_username'
+                               'transact_by_username', 'is_reward_redeemed', 'is_sales_transaction', 'allow_to_revert',
                                ]
     
     def to_transaction_details_json(self):
@@ -80,7 +89,8 @@ class CustomerTransaction(BaseNModel, DictModel):
     
     @property
     def transact_outlet_key(self):
-        return self.transact_outlet.urlsafe()
+        if self.transact_outlet:
+            return self.transact_outlet.urlsafe()
     
     @property
     def transact_merchant_acct_key(self):
@@ -92,23 +102,40 @@ class CustomerTransaction(BaseNModel, DictModel):
     
     @property
     def transact_outlet_details(self):
-        return Outlet.fetch(self.transact_outlet.urlsafe())
+        if self.transact_outlet:
+            return Outlet.fetch(self.transact_outlet.urlsafe())
     
     @property
     def transact_by_user(self):
-        return MerchantUser.fetch(self.transact_by.urlsafe())
+        if self.transact_by:
+            return MerchantUser.fetch(self.transact_by.urlsafe())
     
     @staticmethod
-    def create_manual_transaction(customer, remarks=None, transact_outlet=None, transact_by=None, transact_datetime=None):
+    def update_transaction_reward_have_been_redeemed(transaction_id):
+        
+        logger.debug('Update customer transaction reward have been redeemed by transaction id=%s', transaction_id)
+        
+        if transaction_id:
+            customer_transaction = CustomerTransaction.get_by_transaction_id(transaction_id)
+            if customer_transaction:
+                customer_transaction.is_reward_redeemed = True
+                customer_transaction.put()
+    
+    @staticmethod
+    def create_manual_transaction(customer, remarks=None, transact_outlet=None, transact_by=None, transact_datetime=None, is_sales_transaction=False, allow_to_revert=True): 
         return CustomerTransaction.create_system_transaction(customer, remarks=remarks, transact_outlet=transact_outlet, transact_by=transact_by, 
                                    transact_datetime=transact_datetime, reward_giveaway_method=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_MANUAL,
+                                   is_sales_transaction = is_sales_transaction, allow_to_revert=allow_to_revert,
                                    )
     
     @staticmethod
     def create_system_transaction(customer, transact_amount=.0, tax_amount=.0, invoice_id=None, remarks=None, 
                transact_outlet=None, transact_by=None, transact_datetime=None, reward_giveaway_method=program_conf.PROGRAM_REWARD_GIVEAWAY_METHOD_SYSTEM,
+               is_sales_transaction = False, allow_to_revert=True
                ):
         
+        transact_by_username = None
+        
         if is_not_empty(transact_by):
             if isinstance(transact_by, MerchantUser):
                 transact_by_username = transact_by.username
@@ -117,9 +144,7 @@ class CustomerTransaction(BaseNModel, DictModel):
         transaction_id = generate_transaction_id()
         
         if transact_datetime is None:
-            transact_datetime = datetime.now()
-            if HANDLE_DATETIME_WITH_GMT:
-                transact_datetime = transact_datetime - timedelta(hours=int(SERVER_DATETIME_GMT))
+            transact_datetime = datetime.utcnow()
         
         logger.debug('generated transaction_id=%s', transaction_id)
         logger.debug('invoice_id=%s', invoice_id)
@@ -128,13 +153,11 @@ class CustomerTransaction(BaseNModel, DictModel):
         logger.debug('transact_datetime=%s', transact_datetime)
         logger.debug('transact_by_username=%s', transact_by_username)
         
-        customer.last_transact_datetime = transact_datetime
-        
         customer_transaction = CustomerTransaction(
                                                     parent                  = customer.create_ndb_key(),
                                                     
                                                     transact_merchant       = customer.registered_merchant_acct.create_ndb_key(),
-                                                    transact_outlet         = transact_outlet.create_ndb_key(),
+                                                    transact_outlet         = transact_outlet.create_ndb_key() if transact_outlet else None,
                                                     
                                                     tax_amount              = tax_amount,
                                                     transact_amount         = transact_amount,
@@ -143,11 +166,14 @@ class CustomerTransaction(BaseNModel, DictModel):
                                                     invoice_id              = invoice_id,
                                                     remarks                 = remarks,
                                                     
-                                                    transact_by             = transact_by.create_ndb_key(),
+                                                    transact_by             = transact_by.create_ndb_key() if transact_by else None,
                                                     transact_by_username    = transact_by_username,
                                                     
                                                     transact_datetime       = transact_datetime,
                                                     reward_giveaway_method  = reward_giveaway_method,
+                                                    
+                                                    is_sales_transaction    = is_sales_transaction,
+                                                    allow_to_revert         = allow_to_revert,
                                                     )
         
         customer_transaction.put()
@@ -164,6 +190,8 @@ class CustomerTransaction(BaseNModel, DictModel):
         
         entiteld_vouchers_list          = CustomerEntitledVoucher.list_by_transaction_id(transaction_id)
         
+        entitled_prepaid_list           = CustomerPrepaidReward.list_by_transaction_id(transaction_id)
+        
         is_transaction_reward_used      = False
         
         customer_reward_summary     = customer_acct.reward_summary or {}
@@ -232,7 +260,7 @@ class CustomerTransaction(BaseNModel, DictModel):
                     p.reverted_by_username          = reverted_by.username
                     p.put()
                     
-                    entitled_voucher_summary = update_customer_entiteld_voucher_summary_after_reverted(entitled_voucher_summary, p)
+                    entitled_voucher_summary = update_customer_entiteld_voucher_summary_after_reverted_voucher(entitled_voucher_summary, p)
             
             updated_voucher_summary = {}
             
@@ -366,11 +394,26 @@ class CustomerTransactionWithRewardDetails(object):
         self.reward_format                  = reward_details.reward_format
         self.reward_amount                  = reward_details.reward_amount
         self.expiry_date                    = reward_details.expiry_date
-        self.rewarded_datetime              = reward_details.rewarded_datetime
+        self.rewarded_datetime              = reward_details.rewarded_datetime 
         self.reward_format_key              = reward_details.reward_format_key
         
     
+class CustomerTransactionWithPrepaidDetails(object):    
     
+    def __init__(self, transaction_details, prepaid_details):
+        self.transact_customer_key          = transaction_details.transact_customer_key
+        self.transact_merchant_acct_key     = transaction_details.transact_merchant_acct_key
+        self.transact_outlet_key            = transaction_details.transact_outlet_key
+        self.transact_datetime              = transaction_details.transact_datetime
+        self.transaction_id                 = transaction_details.transaction_id
+        self.transact_amount                = transaction_details.transact_amount
+        self.is_revert                      = transaction_details.is_revert
+        self.reverted_datetime              = transaction_details.reverted_datetime
+        self.topup_amount                   = prepaid_details.topup_amount
+        self.prepaid_amount                 = prepaid_details.prepaid_amount
+        self.expiry_date                    = prepaid_details.expiry_date
+        self.topup_datetime                 = prepaid_details.topup_datetime 
+            
     
     
         
