@@ -6,7 +6,7 @@ Created on 10 Apr 2020
 from google.cloud import ndb
 from trexmodel.models.datastore.ndb_models import BaseNModel, DictModel
 #from trexlib.utils.common.cache_util import cache
-from trexlib.utils.string_util import random_number
+from trexlib.utils.string_util import random_number, is_not_empty
 from flask_login import UserMixin
 import logging  
 from trexlib.utils.security_util import generate_user_id, hash_password
@@ -15,6 +15,8 @@ from trexlib import conf as lib_conf
 from trexmodel import conf as model_conf
 import json
 from json import JSONEncoder
+from flask_babel import gettext
+from trexlib.utils.common.date_util import to_day_of_year
 
 logger = logging.getLogger("model")
 
@@ -57,6 +59,19 @@ class UserMin(BaseNModel, DictModel, UserMixin):
     def is_admin_user(self):
         return False
     
+    def is_valid_password(self, checking_password):
+        hashed_signin_password = hash_password(self.user_id, checking_password)
+        
+        logger.debug('is_valid_password: checking_password=%s', checking_password)
+        logger.debug('is_valid_password: hashed_signin_password=%s', hashed_signin_password)
+        logger.debug('is_valid_password: user password=%s', self.password)
+        
+        is_signin_password_valid = hashed_signin_password == self.password
+        
+        logger.debug('is_valid_password: is_signin_password_valid=%s', is_signin_password_valid)
+        
+        return is_signin_password_valid
+    
 class Role(BaseNModel):
     id              = ndb.StringProperty(required=True)
     name            = ndb.StringProperty(required=True)
@@ -80,6 +95,7 @@ class UserBase(UserMin):
     #---------------------------------------------------------------------------
     birth_date                  = ndb.DateProperty(required=False, indexed=False) 
     birth_date_date_str         = ndb.StringProperty(required=False)
+    birth_day_in_year           = ndb.IntegerProperty(required=False)
     gender                      = ndb.StringProperty(required=False, choices=set([
                                                                     model_conf.GENDER_MALE_CODE, 
                                                                     model_conf.GENDER_FEMALE_CODE
@@ -131,12 +147,17 @@ class UserBase(UserMin):
         user_id             = generate_user_id()
         reference_code      = random_number(16) 
         birth_date_date_str = None
+        birth_day_in_year   = 0
         
         if birth_date:
             birth_date_date_str = birth_date.strftime('%d/%m')
+            birth_day_in_year   = to_day_of_year(birth_date)
+            
+        if mobile_phone:
+            mobile_phone = mobile_phone.replace(" ", "")    
         
         created_user = cls(user_id=user_id, name=name, email=email, mobile_phone=mobile_phone, 
-                           gender=gender, birth_date=birth_date, birth_date_date_str=birth_date_date_str, 
+                           gender=gender, birth_date=birth_date, birth_date_date_str=birth_date_date_str,  birth_day_in_year=birth_day_in_year,
                            reference_code=reference_code
                            )
         
@@ -147,6 +168,30 @@ class UserBase(UserMin):
         
         return created_user
     
+    @classmethod
+    def update(cls, user_acct=None, **kwargs):
+        logger.debug('**kwargs=%s', kwargs)
+        
+        mobile_phone    = kwargs.get('mobile_phone')
+        password        = kwargs.get('password')
+        
+        if mobile_phone:
+            mobile_phone = mobile_phone.replace(" ", "")
+        
+        kwargs['mobile_phone'] = mobile_phone
+        
+        for key, value in kwargs.items():
+            setattr(user_acct, key, value)
+        
+        if password:
+            hashed_password = hash_password(user_acct.user_id, password)
+            user_acct.password = hashed_password
+        
+        user_acct.put()
+        
+        return user_acct
+        
+    
     @classmethod
     def update_contact(cls, user_acct, address=None, city=None, state=None, postcode=None, country=None):
         user_acct.address    = address
@@ -160,16 +205,17 @@ class UserBase(UserMin):
     def update_biodata(cls, user_acct, gender=None, birth_date=None):
         user_acct.gender        = gender
         user_acct.birth_date    = birth_date
-        birth_date_date_str     = None
+        birth_day_in_year   = 0
+        
         if birth_date:
             birth_date_date_str = birth_date.strftime('%d/%m')
+            birth_day_in_year   = to_day_of_year(birth_date)
         
-        user_acct.birth_date_date_str    = birth_date_date_str
+        user_acct.birth_date_date_str   = birth_date_date_str
+        user_acct.birth_day_in_year     = birth_day_in_year
         
         user_acct.put()    
         
-    
-    
 class User(UserBase):
     redeem_pin               = ndb.StringProperty(required=False)
     
